/* 
 *  Copyright (c) 2005 Saveliy Tretiakov (saveliyt@mail.ru)  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
/* TEST DESCRIPTION
 * Use ansi fuctions to read contents of "System" log.
 * Use unicode fuctions to read contents of "Application" log. 
 */

#include <stdio.h>
#include "eventlogrpc.h" 

#define LOGNAME "System"

VOID PRINT_RECORDA(PEVENTLOGRECORD pRec);
VOID PRINT_RECORDW(PEVENTLOGRECORD pRec);
VOID EventTimeToSystemTime(DWORD EventTime, SYSTEMTIME *pSystemTime);

int main()
{
	NTSTATUS result;
	UNICODE_STRING StrNull;
	HANDLE hLog;
	ULONG RecordsNum, OldestRec, BytesNeeded, BytesRead;
	ANSI_STRING LogName;
	PBYTE Buf;
	UINT i;

	LogName.Length = strlen(LOGNAME);
	LogName.MaximumLength = strlen(LOGNAME);
	LogName.Buffer = LOGNAME;
	
	StrNull.Length = 0;
	StrNull.MaximumLength = 0;
	StrNull.Buffer = NULL;

	RpcTryExcept
	{
		printf("\n==================== ANSI FUNCTIONS TEST =====================\n");
		
		hLog = 0;
		result = EventLogOpenA("\\\\.",  &LogName, (PANSI_STRING)&StrNull, 1, 1, &hLog);
		printf("EventLogOpenA() returned %x, hLog = %x\n", result, hLog);
		
		result = EventLogRecordsNumber(hLog, &RecordsNum);
		printf("EventLogRecordsNumber() returned %x, RecordsNum=%d\n", result, RecordsNum);
		
		result = EventLogGetOldestRec(hLog, &OldestRec);
		printf("EventLogGetOldestRec() returned %x, OldestRec=%d\n", result, OldestRec);
		
		for(i = OldestRec; i <= RecordsNum; i++)
		{
			result = EventLogReadA(hLog,  EVENTLOG_FORWARDS_READ | EVENTLOG_SEQUENTIAL_READ,
									0, 0, (PBYTE)&BytesRead, &BytesRead, &BytesNeeded);
			/* printf("EventLogReadA() returned %x, BytesNeeded=%d\n", 
							result, BytesNeeded); */
		
			Buf = (PBYTE)malloc(BytesNeeded);
			result = EventLogReadA(hLog,  EVENTLOG_FORWARDS_READ | EVENTLOG_SEQUENTIAL_READ,
									0, BytesNeeded, Buf, &BytesRead, &BytesNeeded);
			/* printf("EventLogReadA() returned %x, BytesRead=%d, BytesNeeded=%d\n", 
							result, BytesRead, BytesNeeded);  */
			free(Buf);
		
			printf("\n---- RECORD %d ----\n", ((PEVENTLOGRECORD)Buf)->RecordNumber);
			PRINT_RECORDA((PEVENTLOGRECORD)Buf);
		}
		
		EventLogClose(&hLog);
		
		printf("\n\n================== UNICODE FUNCTIONS TEST ====================\n");
		
		/* BUG BUG BUG BUG BUG BUG BUG BUG BUG BUG BUG BUG 
		   UNICODE_STRING initialized with nulls works, 
		   structure initialized with other values doesn't.  */		
		
		hLog = 0;
		result = EventLogOpenW(L"\\\\.", &StrNull, &StrNull, 1, 1, &hLog);
		printf("EventLogOpenW() returned %x, hLog = %x\n", result, hLog);

		result = EventLogRecordsNumber(hLog, &RecordsNum);
		printf("EventLogRecordsNumber() returned %x, RecordsNum=%d\n", result, RecordsNum);
		
		result = EventLogGetOldestRec(hLog, &OldestRec);
		printf("EventLogGetOldestRec() returned %x, OldestRec=%d\n", result, OldestRec);

		for(i = OldestRec; i <= RecordsNum; i++)
		{
			result = EventLogReadW(hLog,  EVENTLOG_FORWARDS_READ | EVENTLOG_SEQUENTIAL_READ,
									0, 0, (PBYTE)&BytesRead, &BytesRead, &BytesNeeded);
			/* printf("EventLogReadW() returned %x, BytesNeeded=%d\n", 
							result, BytesNeeded); */
		
			Buf = (PBYTE)malloc(BytesNeeded);
			result = EventLogReadW(hLog,  EVENTLOG_FORWARDS_READ | EVENTLOG_SEQUENTIAL_READ,
									0, BytesNeeded, Buf, &BytesRead, &BytesNeeded);
			/* printf("EventLogReadW() returned %x, BytesRead=%d, BytesNeeded=%d\n", 
							result, BytesRead, BytesNeeded); */
			free(Buf);
		
			printf("\n---- RECORD %d ----\n", ((PEVENTLOGRECORD)Buf)->RecordNumber);
			PRINT_RECORDW((PEVENTLOGRECORD)Buf);
		}
		
		EventLogClose(&hLog);	
	}
	RpcExcept(1) 
	{
		printf("Runtime reported exception %ld\n", RpcExceptionCode());
	}
	RpcEndExcept

	return 0;
}

VOID PRINT_RECORDA(PEVENTLOGRECORD pRec)
{
	UINT i;
	CHAR *str;
	SYSTEMTIME time;
	
	printf("Length=%d\n", pRec->Length );
	printf("Reserved=%x\n", pRec->Reserved );
	printf("RecordNumber=%d\n", pRec->RecordNumber );
	
	EventTimeToSystemTime(pRec->TimeGenerated, &time);
	printf("TimeGenerated=%d.%d.%d %d:%d:%d\n", 
			time.wDay, time.wMonth, time.wYear,
			time.wHour, time.wMinute, time.wSecond);

	EventTimeToSystemTime(pRec->TimeWritten, &time);  
	printf("TimeWritten=%d.%d.%d %d:%d:%d\n", 
			time.wDay, time.wMonth, time.wYear,
			time.wHour, time.wMinute, time.wSecond);

	printf("EventID=%d\n", pRec->EventID ); 

	printf("EventType= ");
	switch(pRec->EventType)
	{
		case EVENTLOG_ERROR_TYPE:
			printf("EVENTLOG_ERROR_TYPE\n");
			break;
		case EVENTLOG_WARNING_TYPE:
			printf("EVENTLOG_WARNING_TYPE\n");
			break;
		case EVENTLOG_INFORMATION_TYPE:
			printf("EVENTLOG_INFORMATION_TYPE\n");
 			break;
		case EVENTLOG_AUDIT_SUCCESS:
			printf("EVENTLOG_AUDIT_SUCCESS\n");
			break;
		case EVENTLOG_AUDIT_FAILURE:
			printf("EVENTLOG_AUDIT_FAILURE\n");
			break;
		default:
			printf("%x\n");
	}	

	printf("NumStrings=%d\n",  pRec->NumStrings );
	printf("EventCategory=%d\n",  pRec->EventCategory); 
	printf("ReservedFlags=%x\n", pRec->ReservedFlags);
	printf("ClosingRecordNumber=%d\n", pRec->ClosingRecordNumber);
	printf("StringOffset=%d\n", pRec->StringOffset); 
	printf("UserSidLength=%d\n", pRec->UserSidLength);  
	printf("UserSidOffset=%d\n", pRec->UserSidOffset); 
	printf("DataLength=%d\n", pRec->DataLength); 
	printf("DataOffset=%d\n", pRec->DataOffset); 

	printf("SourceName: %s\n", (CHAR *)(((PBYTE)pRec)+sizeof(EVENTLOGRECORD)));
	i = (lstrlenA((CHAR *)(((PBYTE)pRec)+sizeof(EVENTLOGRECORD)))+1);
	printf("ComputerName: %s\n", (CHAR *)(((PBYTE)pRec)+sizeof(EVENTLOGRECORD)+i));
	if(pRec->NumStrings){
		printf("Strings:\n");
		str = (CHAR*)(((PBYTE)pRec)+pRec->StringOffset);
		for(i = 0; i < pRec->NumStrings; i++)
		{
			printf("[%d] %s\n", i, str);
			str = str+lstrlenA(str)+1;
		}
	}
}

VOID PRINT_RECORDW(PEVENTLOGRECORD pRec)
{
	UINT i;
	WCHAR *str;
	SYSTEMTIME time;
	
	printf("Length=%d\n", pRec->Length );
	printf("Reserved=%x\n", pRec->Reserved );
	printf("RecordNumber=%d\n", pRec->RecordNumber );
	
	EventTimeToSystemTime(pRec->TimeGenerated, &time);
	printf("TimeGenerated=%d.%d.%d %d:%d:%d\n", 
			time.wDay, time.wMonth, time.wYear,
			time.wHour, time.wMinute, time.wSecond);

	EventTimeToSystemTime(pRec->TimeWritten, &time);  
	printf("TimeWritten=%d.%d.%d %d:%d:%d\n", 
			time.wDay, time.wMonth, time.wYear,
			time.wHour, time.wMinute, time.wSecond);

	printf("EventID=%d\n", pRec->EventID ); 

	printf("EventType= ");
	switch(pRec->EventType)
	{
		case EVENTLOG_ERROR_TYPE:
			printf("EVENTLOG_ERROR_TYPE\n");
			break;
		case EVENTLOG_WARNING_TYPE:
			printf("EVENTLOG_WARNING_TYPE\n");
			break;
		case EVENTLOG_INFORMATION_TYPE:
			printf("EVENTLOG_INFORMATION_TYPE\n");
 			break;
		case EVENTLOG_AUDIT_SUCCESS:
			printf("EVENTLOG_AUDIT_SUCCESS\n");
			break;
		case EVENTLOG_AUDIT_FAILURE:
			printf("EVENTLOG_AUDIT_FAILURE\n");
			break;
		default:
			printf("%x\n");
	}	

	printf("NumStrings=%d\n",  pRec->NumStrings );
	printf("EventCategory=%d\n",  pRec->EventCategory); 
	printf("ReservedFlags=%x\n", pRec->ReservedFlags);
	printf("ClosingRecordNumber=%d\n", pRec->ClosingRecordNumber);
	printf("StringOffset=%d\n", pRec->StringOffset); 
	printf("UserSidLength=%d\n", pRec->UserSidLength);  
	printf("UserSidOffset=%d\n", pRec->UserSidOffset); 
	printf("DataLength=%d\n", pRec->DataLength); 
	printf("DataOffset=%d\n", pRec->DataOffset); 

	printf("SourceName: %S\n", (WCHAR *)(((PBYTE)pRec)+sizeof(EVENTLOGRECORD)));
	i = (lstrlenW((WCHAR *)(((PBYTE)pRec)+sizeof(EVENTLOGRECORD)))+1)*sizeof(WCHAR);
	printf("ComputerName: %S\n", (WCHAR *)(((PBYTE)pRec)+sizeof(EVENTLOGRECORD)+i));
	if(pRec->NumStrings){
		printf("Strings:\n");
		str = (WCHAR*)(((PBYTE)pRec)+pRec->StringOffset);
		for(i = 0; i < pRec->NumStrings; i++)
		{
			printf("[%d] %S\n", i, str);
			str = str+lstrlenW(str)+1;
		}
	}
}

VOID EventTimeToSystemTime(DWORD EventTime, SYSTEMTIME *pSystemTime)
{
	SYSTEMTIME st1970 = { 1970, 1, 0, 1, 0, 0, 0, 0 };
	FILETIME ftLocal;
	union {
		FILETIME ft;
		LONGLONG ll;
	} u1970, uUCT;
	
	uUCT.ft.dwHighDateTime = 0;
	uUCT.ft.dwLowDateTime = EventTime;
	SystemTimeToFileTime(&st1970, &u1970.ft);
	uUCT.ll = uUCT.ll * 10000000 + u1970.ll;
	FileTimeToLocalFileTime(&uUCT.ft, &ftLocal);
	FileTimeToSystemTime(&ftLocal, pSystemTime);
}
